@@ -47,6 +47,7 @@ from codalab.lib.print_util import pretty_print_json
 from codalab.lib.upload_manager import UploadManager
 from codalab.lib import formatting
 from codalab.model.worker_model import WorkerModel
+from codalab.lib.beam.filesystems import AZURE_BLOB_ACCOUNT_NAME
 
 MAIN_BUNDLE_SERVICE = 'https://worksheets.codalab.org'
 
@@ -249,7 +250,7 @@ class CodaLabManager(object):
         """
         store_type = self.config.get('bundle_store', 'MultiDiskBundleStore')
         if store_type == MultiDiskBundleStore.__name__:
-            return MultiDiskBundleStore(self.codalab_home)
+            return MultiDiskBundleStore(self.model(), self.codalab_home, AZURE_BLOB_ACCOUNT_NAME)
         else:
             print("Invalid bundle store type \"%s\"", store_type, file=sys.stderr)
             sys.exit(1)
