@@ -8,6 +8,7 @@ import shutil
 import tarfile
 import tempfile
 import logging
+from typing import IO, Union, cast
 
 from codalab.common import UsageError
 from codalab.worker.file_util import (
@@ -47,11 +48,14 @@ def strip_archive_ext(path):
     raise UsageError('Not an archive: %s' % path)
 
 
-def unpack(ext, source, dest_path):
-    """
-    Unpack the archive |source| to |dest_path|.
-    Note: |source| can be a file handle or a path.
-    |ext| contains the extension of the archive.
+def unpack(ext: str, source: Union[str, IO[bytes]], dest_path: str):
+    """Unpack the archive |source| to |dest_path|.
+
+    Args:
+        ext (str): Extension of the archive.
+        source (Union[str, IO[bytes]]): File handle or path to the source.
+        dest_path ([type]): Destination path to unpack to.
+
     """
     close_source = False
     try:
@@ -77,7 +81,7 @@ def unpack(ext, source, dest_path):
         raise UsageError('Invalid archive upload.')
     finally:
         if close_source:
-            source.close()
+            cast(IO[bytes], source).close()
 
 
 def pack_files_for_upload(
@@ -108,8 +112,7 @@ def pack_files_for_upload(
         'fileobj': <file object of archive>,
         'filename': <name of archive file>,
         'filesize': <size of archive in bytes, or None if unknown>,
-        'should_unpack': <True iff archive should be unpacked at server>,
-        'should_simplify': <True iff directory should be 'simplified' at server>
+        'should_unpack': <True iff archive should be unpacked at server>
         }
     """
     exclude_patterns = exclude_patterns or []
@@ -143,7 +146,6 @@ def pack_files_for_upload(
                 'filename': filename + '.tar.gz',
                 'filesize': None,
                 'should_unpack': True,
-                'should_simplify': False,
             }
         elif path_is_archive(source):
             return {
@@ -151,7 +153,6 @@ def pack_files_for_upload(
                 'filename': filename,
                 'filesize': os.path.getsize(source),
                 'should_unpack': should_unpack,
-                'should_simplify': True,
             }
         elif force_compression:
             return {
@@ -159,7 +160,6 @@ def pack_files_for_upload(
                 'filename': filename + '.gz',
                 'filesize': None,
                 'should_unpack': True,
-                'should_simplify': False,
             }
         else:
             return {
@@ -167,7 +167,6 @@ def pack_files_for_upload(
                 'filename': filename,
                 'filesize': os.path.getsize(source),
                 'should_unpack': False,
-                'should_simplify': False,
             }
 
     # Build archive file incrementally from all sources
@@ -188,17 +187,8 @@ def pack_files_for_upload(
         return None if should_exclude(tarinfo.name) else tarinfo
 
     for source in sources:
-        if should_unpack and path_is_archive(source):
-            # Unpack archive into scratch space
-            dest_basename = strip_archive_ext(os.path.basename(source))
-            dest_path = os.path.join(scratch_dir, dest_basename)
-            unpack(get_archive_ext(source), source, dest_path)
-
-            # Add file or directory to archive
-            archive.add(dest_path, arcname=dest_basename, recursive=True)
-        else:
-            # Add file to archive, or add files recursively if directory
-            archive.add(source, arcname=os.path.basename(source), recursive=True, filter=filter)
+        # Add file to archive, or add files recursively if directory
+        archive.add(source, arcname=os.path.basename(source), recursive=True, filter=filter)
 
     # Clean up, rewind archive file, and return it
     archive.close()
@@ -210,5 +200,4 @@ def pack_files_for_upload(
         'filename': 'contents.tar.gz',
         'filesize': filesize,
         'should_unpack': True,
-        'should_simplify': False,
     }
