@@ -109,9 +109,7 @@ def check_run_permission(bundle):
     Checks whether the current user can run the bundle.
     """
     if not check_bundle_have_run_permission(local.model, request.user, bundle):
-        abort(
-            http.client.FORBIDDEN, "User does not have permission to run bundle.",
-        )
+        abort(http.client.FORBIDDEN, "User does not have permission to run bundle.")
 
 
 @post(
